"""Keeps track of the network's values"""
import numpy as np
class Layer:
    def __init__(self):
        #size of network for weight generation
        self.Layers = []

    def pop(self):
        self.Layers = []


class Kernel:
    def __init__(self):
        self.Kernels = np.array([])

    def pop(self):
        self.Kernels = np.array([])


class Bias:
    def __init__(self):
        self.Biases = np.array([])

    def pop(self):
        self.Biases = np.array([])



class Weight:
    def __init__(self):
        self.Weights = []

    def pop(self):
        self.Weights = []

    def add(self, val):
        self.Weights.append(val)


class Neuron:
    def __init__(self):
        self.Neurons = []

    def pop(self):
        self.Neurons = []

    def add(self, val):
        self.Neurons.append(val)


class Derivative:
    def __init__(self):
        self.Derivatives = []

    def pop(self):
        self.Derivatives = []

    def add(self, val):
        self.Derivatives.append(val)


#Spacific use cases (note to self, add condition statements)

class Velocity:
    def __init__(self):
        self.Velocities = None
    def pop(self):
        self.Velocities = []

#GLOBAL NETWORK VALUES
WeightVals = Weight()
KernelVals = Kernel()
LayerVals = Layer()
NeuronVals = Neuron()
DerivativeVals = Derivative()
# LOCAL NETWORK VALUES
VelocityVals = Velocity()
ActivationList = []
ActivationDerivativeList = []
ActivationListString = []
ActivationDerivativeListString = []
OptimizerString = ['gradient decent']
LossString = ['mean square error']
Loss = [[]]
Optimizer = []



from deeprai.engine.cython import loss as loss_function
class MeanAbsoluteError:
    def mean_absolute_error(self, outputs, targets):
        return loss_function.mean_absolute_error(outputs, targets)


from deeprai.engine.cython import loss as loss_function
class CategoricalCrossEntropy:
    def categorical_cross_entropy(self, outputs, targets):
        return loss_function.categorical_cross_entropy(outputs, targets)


from deeprai.engine.cython import activation as act
class Softmax:
    def softmax(self, neuron):
        return act.softmax(neuron)

    def softmax_derivative(self, softmax):
        return act.softmax_derivative(softmax)



from deeprai.engine.cython import activation as act

class Tanh:
    def tanh(self, neuron):
        return act.tanh(neuron)

    def tanh_derivative(self, tanh):
        return act.tanh_derivative(tanh)



import deeprai.engine.build_model as builder
from deeprai.models.feed_forward.feed_forward import FeedForward
from deeprai.models.convolutional.convolutional import Convolutional
from deeprai.tools.file_manager.save import Save
from deeprai.tools.file_manager.load import Load

class Convolutional(Convolutional):
    def __init__(self):
        self.spawn = builder.Build()
        super().__init__()

    def update_network(self):
        pass

    def clear_global_network(self):
        pass

    def clear_local_network(self):
        self.layers = []
        self.bias = []
        self.kernels = []


class FeedForward(FeedForward):
    def __init__(self):
        self.spawn = builder.Build()
        super().__init__()

    def save(self, file_location):
        file = Save(file_location)
        file.save()

    def load(self, file_location):
        Load(file_location)

    def update_network(self):
        self.base_weights.update(self.weights)
        self.base_bias.update(self.bias)

    def clear_global_network(self):
        self.base_weights.pop()
        self.base_bias.pop()

    def clear_local_network(self):
        self.weights = []
        self.bias = []

